﻿// ------------------------------------------------------------------------------------
//
// BinaryInput.CS                                                Ulrich Müller 22.12.09
//                                                               Aspire C# 2005 Express
// Assembly FishFaceTX.DLL : Klasse BinaryInput
// Basis der Digitalen-Eingänge
// D5KInput mit PushButton, PhotoTransistor, ReedContact
// HalfTrack : BinaryInput
//
// ------------------------------------------------------------------------------------
using System;
using System.Collections.Generic;
using System.Text;
using System.Threading;

namespace FishFaceTX {
  public abstract class BinaryInput : DeviceBase {
    public delegate void Changed(BinaryInput sender);
    public event Changed ChangedToTrue;
    public event Changed ChangedToFalse;

    private bool actValue, oldValue;

    public BinaryInput(FishControl fc, Ctr ctrId, Unv InputName) 
      : this(fc, false, ctrId, InputName) {
    }
    public BinaryInput(FishControl fc, bool WithEvents, Ctr ctrId, Unv InputName)
      : base(fc, WithEvents, (int)ctrId, (int)InputName) {
      base.deviceType = "D5K";
      base.threadInterval = 22;
    }

    protected override void EventLoop() {
      oldValue = ActualValue;
      while (!tx.NotHalt && !tx.IsESC) {
        actValue = ActualValue;
        if (actValue != oldValue)
          if (actValue && ChangedToTrue != null) ChangedToTrue(this);
          else if (ChangedToFalse != null) ChangedToFalse(this);
        oldValue = actValue;
        Thread.Sleep(base.threadInterval);
      }
    }

    public override string ConnectorName {
      get {
        return ((Unv)base.connectorNumber).ToString();
      }
    }
    public abstract bool ActualValue {
      get;
    }

    public bool IsFalse {
      get {
        return !ActualValue;
      }
    }
    public bool IsTrue {
      get {
        return ActualValue;
      }
    }

    public void WaitForTrue() {
      tx.WaitForInput((Unv)base.connectorNumber, true);
    }
    public void WaitForFalse() {
      tx.WaitForInput((Unv)base.connectorNumber, false);
    }
    public void WaitForHigh() {
      tx.WaitForHigh((Unv)base.connectorNumber);
    }
    public void WaitForLow() {
      tx.WaitForLow((Unv)base.connectorNumber);
    }
  }

  public class D5KInput : BinaryInput {
    public D5KInput(FishControl fc, Ctr ctrId, Unv InputName)
      : this(fc, false, ctrId, InputName) {
    }
    public D5KInput(FishControl fc, bool WithEvents, Ctr ctrId, Unv InputName)
      : base(fc, WithEvents, ctrId, InputName) {
      base.deviceType = "D5K";
    }
    public override bool ActualValue {
      get {
        return tx.GetInput((Ctr)base.ControllerNumber, (Unv)connectorNumber);
      }
    }
  }
  public class PushButton : D5KInput {
    public PushButton(FishControl fc, Ctr ctrId, Unv InputName)
      : this(fc, false, ctrId, InputName) {
    }
    public PushButton(FishControl fc, bool WithEvents, Ctr ctrId, Unv InputName)
      : base(fc, WithEvents, ctrId, InputName) {
    }
  }
  public class PhotoTransistor : D5KInput {
    public PhotoTransistor(FishControl fc, Ctr ctrId, Unv InputName)
      : this(fc, false, ctrId, InputName) {
    }
    public PhotoTransistor(FishControl fc, bool WithEvents, Ctr ctrId, Unv InputName)
      : base(fc, WithEvents, ctrId, InputName) {
    }
  }
  public class ReedContact : D5KInput {
    public ReedContact(FishControl fc, Ctr ctrId, Unv InputName)
      : this(fc, false, ctrId, InputName) {
    }
    public ReedContact(FishControl fc, bool WithEvents, Ctr ctrId, Unv InputName)
      : base(fc, WithEvents, ctrId, InputName) {
    }
  }

  public class HalfTrack : BinaryInput {
    public HalfTrack(FishControl fc, Ctr ctrId, Unv InputName)
      : this(fc, false, ctrId, InputName) {
    }
    public HalfTrack(FishControl fc, bool WithEvents, Ctr ctrId, Unv InputName)
      : base(fc, WithEvents, ctrId, InputName) {
      base.deviceType = "D10V";
    }
    public override bool ActualValue {
      get {
        return tx.GetTrack((Ctr)base.controllerNumber, (Unv)connectorNumber);
      }
    }
  }
}