﻿// ------------------------------------------------------------------------------------
//
// DeviceBase.CS                                                 Ulrich Müller 29.10.09
//                                                               Aspire C# 2005 Express
// Assembly FishFaceTX.DLL : Klasse DeviceBase
// Basis der Device-Klassen
//
// ------------------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Text;
using System.Threading;

namespace FishFaceTX {
  public abstract class DeviceBase {
    internal  FishFace tx;
    protected int      controllerNumber;
    protected string   controllerName;
    protected int      connectorNumber;
    protected string   deviceType;
    protected string   deviceText;
    protected bool     withEvents;
    protected Thread   deviceThread;
    protected int      threadInterval;

    public DeviceBase(FishControl fc, bool WithEvents, int ControllerNumber,
      int ConnectorNumber) {
      this.controllerNumber   = ControllerNumber;
      this.controllerName     = ((Ctr)ControllerNumber).ToString();
      this.connectorNumber    = ConnectorNumber;
      this.deviceType         = "undef";
      this.deviceText         = "-";
      this.withEvents         = WithEvents;
      this.threadInterval     = 11;
      this.deviceThread       = null;
      fc.AddDevice(this);
    }

    public int ControllerNumber {
      get {
        return controllerNumber;
      }
    }
    public string ControllerName {
      get {
        return controllerName;
      }
    }

    public int ConnectorNumber {
      get {
        return connectorNumber;
      }
    }
    public abstract string ConnectorName {
      get;
    }

    public string DeviceType {
      get {
        return deviceType;
      }
    }
    public string DeviceName {
      get {
        string d = base.ToString();
        return d.Substring(d.LastIndexOf('.') + 1);
      }
    }
    public string DeviceText {
      get {
        return deviceText;
      }
      set {
        deviceText = value;
      }
    }
    public override string ToString() {
      return DeviceName + " : " + ControllerName + " - " + ConnectorName;
    }

    public bool WithEvents {
      get {
        return withEvents;
      }
      set {
        withEvents = value;
      }
    }
    public virtual int ThreadInterval {
      get {
        return threadInterval;
      }
      set {
        threadInterval = value < 11 ? 11 : value;
      }
    }

    protected abstract void EventLoop();
    internal bool StartThread() {
      if (withEvents) {
        deviceThread = new Thread(new ThreadStart(EventLoop));
        deviceThread.IsBackground = true;
        deviceThread.Start();
        return true;
      }
      else return false;
    }
    internal void StopThread() {
      if(deviceThread!=null)
      if (withEvents && deviceThread.IsAlive) deviceThread.Abort();
    }
  }
}
