﻿// --------------------------------------------------------------------------------------
//
// Drive2NXT.CS                                                    Ulrich Müller 02.11.09
//                                                                 Aspire C# 2005 Express
// Simultanbetrieb von zwei EncoderMotoren nach dem Objekt Bewegung von Mindstorm NXT
//
// --------------------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Text;

namespace FishFaceTX {
  public enum Steering {
    TurnLeft, BackLeft, StopLeft, SlightLeft,
    Straight,
    SlightRight, StopRight, BackRight, TurnRight
  }

  public class Drive2NXT : DeviceBase {
    EncoderMotor leftMotor;
    EncoderMotor rightMotor;

    public Drive2NXT(FishControl fc, Ctr ctrId,
      Mot LeftMotor, Mot RightMotor)
      : base(fc, false, (int)ctrId, 111) {
      base.deviceType = "Drive2NXT";
      leftMotor       = new EncoderMotor(fc, ctrId, LeftMotor);
      rightMotor      = new EncoderMotor(fc, ctrId, RightMotor);
    }

    protected override void EventLoop() { }

    public override string ConnectorName {
      get {
        return base.DeviceType;
      }
    }
    public int LeftMotorNr {
      get {
        return leftMotor.ConnectorNumber;
      }
    }
    public string LeftMotorName {
      get {
        return leftMotor.ConnectorName;
      }
    }
    public int LeftImpulseSwitchNr {
      get {
        return leftMotor.ImpulseSwitchNr;
      }
    }
    public string LeftImpulseSwitchName {
      get {
        return leftMotor.ImpulseSwitchName;
      }
    }
    public string RightMotorName {
      get {
        return rightMotor.ConnectorName;
      }
    }
    public int RightImpulseSwitchNr {
      get {
        return rightMotor.ImpulseSwitchNr;
      }
    }
    public string RightImpulseSwitchName {
      get {
        return rightMotor.ImpulseSwitchName;
      }
    }

    public void Forward() {
      leftMotor.RunLeft();
      rightMotor.RunLeft();
    }
    public void Backward() {
      leftMotor.RunRight();
      rightMotor.RunRight();
    }
    public void StopRun() {
      leftMotor.StopRun();
      rightMotor.StopRun();
    }
    public void Forward(Steering Direction, int Speed, int Impulse) {
      int HalfSpeed = (Speed - 222) / 2 + 222;
      switch (Direction) {
        case Steering.TurnLeft :
          tx.SetMotor((Ctr)leftMotor.ControllerNumber, (Mot)leftMotor.ConnectorNumber, 
            Dir.Left, Speed);
          rightMotor.Backward(Speed, Impulse);
          tx.SetMotor((Ctr)leftMotor.ControllerNumber, (Mot)leftMotor.ConnectorNumber,
            Dir.Off);
          break;
        case Steering.BackLeft :
          tx.SetMotor((Ctr)rightMotor.ControllerNumber, (Mot)rightMotor.ConnectorNumber,
            Dir.Left, HalfSpeed);
          rightMotor.Forward(Speed, Impulse);
          tx.SetMotor((Ctr)rightMotor.ControllerNumber, (Mot)rightMotor.ConnectorNumber,
            Dir.Off);
          break;
        case Steering.StopLeft :
          tx.SetMotor((Ctr)leftMotor.ControllerNumber, (Mot)leftMotor.ConnectorNumber,
            Dir.Off);
          rightMotor.Forward(Speed, Impulse);
          break;
        case Steering.SlightLeft :
          tx.SetMotor((Ctr)rightMotor.ControllerNumber, (Mot)rightMotor.ConnectorNumber,
            Dir.Left, HalfSpeed);
          leftMotor.Backward(Speed, Impulse);
          tx.SetMotor((Ctr)rightMotor.ControllerNumber, (Mot)rightMotor.ConnectorNumber,
            Dir.Off);
          break;
        case Steering.Straight :
          tx.StartMotor((Ctr)leftMotor.ControllerNumber, (Mot)leftMotor.ConnectorNumber,
            Dir.Left, Speed, Impulse);
          tx.StartMotor((Ctr)rightMotor.ControllerNumber, (Mot)rightMotor.ConnectorNumber,
           Dir.Left, Speed, Impulse);
          tx.WaitForMotors((Mot)leftMotor.ConnectorNumber, 
            (Mot)rightMotor.ConnectorNumber);
          break;
        case Steering.SlightRight :
          tx.SetMotor((Ctr)leftMotor.ControllerNumber, (Mot)leftMotor.ConnectorNumber,
            Dir.Left, HalfSpeed);
          rightMotor.Backward(Speed, Impulse);
          tx.SetMotor((Ctr)leftMotor.ControllerNumber, (Mot)leftMotor.ConnectorNumber,
            Dir.Off);
          break;
        case Steering.StopRight :
          tx.SetMotor((Ctr)rightMotor.ControllerNumber, (Mot)rightMotor.ConnectorNumber,
            Dir.Off);
          leftMotor.Forward(Speed, Impulse);
          break;
        case Steering.BackRight :
          tx.SetMotor((Ctr)leftMotor.ControllerNumber, (Mot)leftMotor.ConnectorNumber,
            Dir.Right, HalfSpeed);
          rightMotor.Forward(Speed, Impulse);
          tx.SetMotor((Ctr)leftMotor.ControllerNumber, (Mot)leftMotor.ConnectorNumber,
            Dir.Off);
          break;
        case Steering.TurnRight :
          tx.SetMotor((Ctr)leftMotor.ControllerNumber, (Mot)leftMotor.ConnectorNumber,
            Dir.Right, Speed);
          rightMotor.Forward(Speed, Impulse);
          tx.SetMotor((Ctr)leftMotor.ControllerNumber, (Mot)leftMotor.ConnectorNumber,
            Dir.Off);
          break;
        default:
          break;
      }
    }
    
    public void Backward(Steering Direction, int Speed, int Impulse) {
      int HalfSpeed = (Speed - 222) / 2 + 222;
      switch (Direction) {
        case Steering.TurnLeft:
          tx.SetMotor((Ctr)leftMotor.ControllerNumber, (Mot)leftMotor.ConnectorNumber,
            Dir.Right, Speed);
          rightMotor.Forward(Speed, Impulse);
          tx.SetMotor((Ctr)leftMotor.ControllerNumber, (Mot)leftMotor.ConnectorNumber,
            Dir.Off);
          break;
        case Steering.BackLeft:
          tx.SetMotor((Ctr)rightMotor.ControllerNumber, (Mot)rightMotor.ConnectorNumber,
            Dir.Left, HalfSpeed);
          rightMotor.Backward(Speed, Impulse);
          tx.SetMotor((Ctr)rightMotor.ControllerNumber, (Mot)rightMotor.ConnectorNumber,
            Dir.Off);
          break;
        case Steering.StopLeft:
          tx.SetMotor((Ctr)rightMotor.ControllerNumber, (Mot)rightMotor.ConnectorNumber,
            Dir.Off);
          rightMotor.Backward(Speed, Impulse);
          break;
        case Steering.SlightLeft:
          tx.SetMotor((Ctr)rightMotor.ControllerNumber, (Mot)rightMotor.ConnectorNumber,
            Dir.Right, HalfSpeed);
          leftMotor.Backward(Speed, Impulse);
          tx.SetMotor((Ctr)rightMotor.ControllerNumber, (Mot)rightMotor.ConnectorNumber,
            Dir.Off);
          break;
        case Steering.Straight:
          tx.StartMotor((Ctr)leftMotor.ControllerNumber, (Mot)leftMotor.ConnectorNumber,
            Dir.Right, Speed, Impulse);
          tx.StartMotor((Ctr)rightMotor.ControllerNumber, (Mot)rightMotor.ConnectorNumber,
           Dir.Right, Speed, Impulse);
          tx.WaitForMotors((Mot)leftMotor.ConnectorNumber,
            (Mot)rightMotor.ConnectorNumber);
          break;
        case Steering.SlightRight:
          tx.SetMotor((Ctr)leftMotor.ControllerNumber, (Mot)leftMotor.ConnectorNumber,
            Dir.Right, HalfSpeed);
          rightMotor.Backward(Speed, Impulse);
          tx.SetMotor((Ctr)leftMotor.ControllerNumber, (Mot)leftMotor.ConnectorNumber,
            Dir.Off);
          break;
        case Steering.StopRight:
          tx.SetMotor((Ctr)leftMotor.ControllerNumber, (Mot)leftMotor.ConnectorNumber,
            Dir.Off);
          leftMotor.Backward(Speed, Impulse);
          break;
        case Steering.BackRight:
          tx.SetMotor((Ctr)leftMotor.ControllerNumber, (Mot)leftMotor.ConnectorNumber,
            Dir.Left, HalfSpeed);
          rightMotor.Backward(Speed, Impulse);
          tx.SetMotor((Ctr)leftMotor.ControllerNumber, (Mot)leftMotor.ConnectorNumber,
            Dir.Off);
          break;
        case Steering.TurnRight:
          tx.SetMotor((Ctr)leftMotor.ControllerNumber, (Mot)leftMotor.ConnectorNumber,
            Dir.Left, Speed);
          rightMotor.Backward(Speed, Impulse);
          tx.SetMotor((Ctr)leftMotor.ControllerNumber, (Mot)leftMotor.ConnectorNumber,
            Dir.Off);
          break;
        default:
          break;
      }
    }    
  }
}
