﻿// --------------------------------------------------------------------------------------
//
// EncoderMotor.CS                                                 Ulrich Müller 01.11.09
//                                                                 Aspire C# 2005 Express
// Motor mit fest zugeordnetem Counter-Eingang an M1 - M4 : C1 - C4
//
// --------------------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Text;

namespace FishFaceTX {
  public class EncoderMotor : DeviceBase {

    public delegate void Counter(EncoderMotor sender, int count);
    public event Counter CountChanged;

    Motor encMotor;
    CounterInput counter;

    public EncoderMotor(FishControl fc, Ctr ctrId, Mot EMotor)
      : this(fc, false, ctrId, EMotor) {
    }
    public EncoderMotor(FishControl fc, bool WithEvents, Ctr ctrId, Mot EMotor)
      : base(fc, WithEvents, (int)ctrId, (int)EMotor) {
      base.deviceType = "EncoderMotor";
      encMotor        = new Motor(fc, ctrId, EMotor);
      counter         = new CounterInput(fc, WithEvents, ctrId, 
                        (Cnt)encMotor.ConnectorNumber);
      if (WithEvents) {
        counter.CurrentValue += new CounterInput.Counter(counter_CountChanged);
      }
    }

    private void counter_CountChanged(CounterInput sender, int count) {
      if (CountChanged != null) CountChanged(this, count);
    }
    protected override void EventLoop() {
    }

    public override string ConnectorName {
      get {
        return base.deviceType;
      }
    }
    public override int ThreadInterval {
      get {
        return base.threadInterval;
      }
      set {
        threadInterval         = value < 44 ? 44 : value;
        counter.ThreadInterval = threadInterval;
      }
    }
    public int ImpulseSwitchNr {
      get {
        return counter.ConnectorNumber;
      }
    }
    public string ImpulseSwitchName {
      get {
        return counter.ConnectorName;
      }
    }

    public void RunLeft() {
      RunLeft(512);
    }
    public void RunLeft(int Speed) {
      tx.SetMotor((Ctr)(base.controllerNumber), (Mot)(base.connectorNumber),
        Dir.Left, Speed);
    }
    public void RunRight() {
      RunRight(512);
    }
    public void RunRight(int Speed) {
      tx.SetMotor((Ctr)(base.controllerNumber), (Mot)(base.connectorNumber),
        Dir.Right, Speed);
    }
    public void StopRun() {
      tx.SetMotor((Ctr)(base.controllerNumber), (Mot)(base.connectorNumber),
        Dir.Off);
    }
    public void Forward(int Speed, int Impulse) {
      StartForward(Speed, Impulse);
      WaitForDone();
    }
    public void StartForward(int Impulse) {
      StartForward(512, Impulse);
    }
    public void StartForward(int Speed, int Impulse) {
      StartLeft(Speed, Impulse);
    }
    public void StartLeft(int Speed, int Impulse) {
      tx.StartMotor((Ctr)(base.controllerNumber), (Mot)(base.connectorNumber),
        Dir.Left, Speed, Impulse);
    }
    public void Backward(int Speed, int Impulse) {
      StartBackward(Speed, Impulse);
      WaitForDone();
    }
    public void StartBackward(int Speed, int Impulse) {
      StartRight(Speed, Impulse);
    }
    public void StartBackward(int Impulse) {
      StartBackward(512, Impulse);
    }
    public void StartRight(int Speed, int Impulse) {
      tx.StartMotor((Ctr)(base.controllerNumber), (Mot)(base.connectorNumber),
        Dir.Right, Speed, Impulse);
    }
    public void StartUp(int Speed, int Impulse) {
      StartForward(Speed, Impulse);
    }
    public void Up(int Speed, int Impulse) {
      Forward(Speed, Impulse);
    }
    public void StartDown(int Speed, int Impulse) {
      StartBackward(Speed, Impulse);
    }
    public void Down(int Speed, int Impulse) {
      Backward(Speed, Impulse);
    }
    public void Go(Dir Dir, int Speed, int Impulse) {
      StartGo(Dir, Speed, Impulse);
      WaitForDone();
    }
    public void StartGo(Dir dir, int Speed, int Impulse) {
      tx.StartMotor((Ctr)(base.controllerNumber), (Mot)(base.connectorNumber),
        dir, Speed, Impulse);
    }
    public void WaitForDone() {
      tx.WaitForMotor((Ctr)(base.controllerNumber), (Mot)(base.connectorNumber));
    }   
  }
}
