﻿// ------------------------------------------------------------------------------------
//
// FishControl.CS                                                Ulrich Müller 29.10.09
//                                                               Aspire C# 2005 Express
// Assembly FishFaceTX.DLL : Klasse FishControl
// Verwaltung der zu einem TX Controller gehörenden Device-Objekte
// public Bereitstellung von FishFace Methoden, die das gesamte Control betreffen
// Weiterreichen der private FishFace-Instanz an die zugehörenden Device-Objekte
//
// ------------------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Runtime.InteropServices;
using System.Text;
using System.Threading;
using System.Windows.Forms;

namespace FishFaceTX {
  public class FishControl {
    private FishFace tx;
    private List<DeviceBase> DeviceList  = new List<DeviceBase>();

    public FishControl() : base() {
      tx = new FishFace();
    }

    public string LibVersion {
      get {
        return FishFace.LibVersion;
      }
    }
    public bool NotHalt {
      get {
        return tx.NotHalt;
      }
      set {
        tx.NotHalt = value;
      }
    }
    public FishFace ControllerInstance {
      get {
        return tx;
      }
    }

    internal void AddDevice(DeviceBase DeviceObject) {
      DeviceList.Add(DeviceObject);
      DeviceObject.tx = this.tx;
    }

    public void Connect(string ComName) {
      try {
        tx.OpenController(ComName);
      }
      catch (FishFaceException) {
        throw (new FishFaceException("ControllerProblem.Connect"));
      }
    }
    public void DisConnect() {
      tx.NotHalt = true;
      foreach (DeviceBase db in DeviceList) if (db.WithEvents) db.StopThread();
      Thread.Sleep(123);
      tx.CloseController();
      Application.DoEvents();
    }
    public void StartEvents() {
      foreach (DeviceBase db in DeviceList) 
        if (db.WithEvents) db.StartThread();
    }

    public void Pause(int mSek) {
      tx.Pause(mSek);
    }
    public bool Finish() {
      return tx.Finish();
    }

    public int DeviceCount {
      get {
        return DeviceList.Count;
      }
    }
    public DeviceBase this[int index] {
      get {
        return DeviceList[index];
      }
    }
  }
}
