﻿// --------------------------------------------------------------------------------------
//
// RobMotor.CS                                                     Ulrich Müller 01.11.09
//                                                                 Aspire C# 2005 Express
// Betrieb eines Motors im Verbund mit einem End- und einem Countereingang
// Die Eingänge sind fest zugeordnet : M1 : I1, C1 ...
// Betrieb asynchron bis WaitForDone
//
// --------------------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Text;
using System.Threading;

namespace FishFaceTX {
  public class RobMotor : DeviceBase {

    public delegate void Position(RobMotor sender, int Pos);
    public event Position ChangedPosition;

    Motor        robMotor;
    D5KInput     endTaster;
    CounterInput impulseTaster;
    private int  maxPos;
    private int  startPos;
    private int  destPos;
    private int  actPos;
    private bool noEvent;
    private Dir  stateValue;

    public RobMotor(FishControl fc, Ctr ctrId, Mot MotorName, int MaxPosition)
      : this(fc, false, ctrId, MotorName, MaxPosition) {
    }
    public RobMotor(FishControl fc, bool WithEvents, 
      Ctr ctrId, Mot MotorName, int MaxPosition)
      : base(fc, WithEvents, (int)ctrId, (int)MotorName) {
      base.deviceType     = "RobMotor";
      this.maxPos         = MaxPosition;
      this.startPos       = 9999;
      this.destPos        = 0;
      base.threadInterval = 11;
      base.withEvents     = WithEvents;
      this.noEvent        = false;

      robMotor      = new Motor(fc, ctrId, MotorName);
      endTaster     = new D5KInput(fc, ctrId, (Unv)((int)MotorName));
      impulseTaster = new CounterInput(fc, ctrId, (Cnt)((int)MotorName));
    }

    protected override void EventLoop() {
      while (!tx.NotHalt && !tx.IsESC) {
        if (noEvent == false) {
          int ac = tx.GetCounter((Ctr)(base.controllerNumber), 
            (Cnt)impulseTaster.ConnectorNumber);
          actPos = startPos < destPos ? startPos + ac : startPos - ac;
          if (ChangedPosition != null) ChangedPosition(this, actPos);
        }
        Thread.Sleep(base.threadInterval);
      }
    }

    public override string ConnectorName {
      get {
        return "RobMotor";
      }
    }
    public int    MotorNr {
      get {
        return robMotor.ConnectorNumber;
      }
    }
    public string MotorName {
      get {
        return robMotor.ConnectorName;
      }
    }
    public int    EndSwitchNr {
      get {
        return endTaster.ConnectorNumber;
      }
    }
    public string EndSwitchName {
      get {
        return endTaster.ConnectorName;
      }
    }
    public int    ImpulseSwitchNr {
      get {
        return impulseTaster.ConnectorNumber;
      }
    }
    public string ImpulseSwitchName {
      get {
        return impulseTaster.ControllerName;
      }
    }
    public int    MaxPosition {
      get {
        return maxPos;
      }
    }
    public int    CurrentPosition {
      get {
        return startPos;
      }
      set {
        startPos = value;
      }
    }
    public int    ActualPosition {
      get {
        return actPos;
      }
    }

    public void StartHome() {
      noEvent  = true;
      stateValue = Dir.On;
      tx.StartRobMotor((Ctr)base.controllerNumber, (Mot)robMotor.ConnectorNumber, 
        Dir.Left, 512, 32111);
      startPos = 0;
      destPos  = 0;
      actPos   = 0;
    }
    public void DriveHome() {
      noEvent  = true;
      stateValue = Dir.On;
      tx.SetMotor((Ctr)base.controllerNumber, (Mot)robMotor.ConnectorNumber, Dir.Left);
      tx.WaitForInput((Ctr)base.controllerNumber, (Unv)robMotor.ConnectorNumber, true);
      tx.SetMotor((Ctr)base.controllerNumber, (Mot)robMotor.ConnectorNumber, Dir.Off);
      startPos = 0;
      destPos  = 0;
      actPos   = 0;
    }
    public void StartDelta(int Inc) {
      this.StartTo(startPos + Inc);
    }
    public void DriveDelta(int Inc) {
      StartDelta(Inc);
      WaitForDone();
    }
    public void StartTo(int Pos) {
      noEvent  = false;
      destPos = Pos > maxPos ? maxPos : Pos;
      if (destPos > startPos) {
        stateValue = Dir.Right;
        tx.StartRobMotor((Ctr)base.controllerNumber, (Mot)robMotor.ConnectorNumber,
          stateValue, 512, destPos - startPos);
      }
      else if (startPos - destPos > 0) {
        stateValue = Dir.Left;
        tx.StartRobMotor((Ctr)base.controllerNumber, (Mot)robMotor.ConnectorNumber,
          stateValue, 512, startPos - destPos);
      }
    }
    public void DriveTo(int Pos) {
      StartTo(Pos);
      WaitForDone();
    }
    public void WaitForDone() {
      int diff = tx.WaitForMotor((Ctr)base.controllerNumber,
        (Mot)robMotor.ConnectorNumber);
      tx.Pause(123);
      tx.ClearCounter((Ctr)base.controllerNumber, (Cnt)robMotor.ConnectorNumber);
      if (stateValue == Dir.Right) actPos = destPos + diff;
      else if (stateValue == Dir.Left) actPos = destPos - diff;
      startPos = actPos;
      noEvent = true;
      stateValue = Dir.Off;
    }
  }
}
