﻿// --------------------------------------------------------------------------------------
//
// RechenTest                                                      Ulrich Müller 07.11.09
//                                                                 Aspire C# 2005 Express
// Kleine Rechenaufgaben, Kontrolle der Lösung über Taschenrechner
//
// Nutzung FishFaceTX.DLL Devices mit Basis umFish50.DLL - ftMscLib.DLL
//
// Belegung Interface :
// X-Achse : M1(EncoderMotor), I1(Endtaster), C1(Impuls)
// Y-Achse : M2(EncoderMotor), I2(Endtaster), C2(Impuls)
// Taster  : M3(XS-Motor), I3(Endtaster)
//
// --------------------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using FishFaceTX;

namespace RechenTest {
  public partial class frmMain : Form {
    FishControl fc = new FishControl();
    RobMotors   Position;
    Betaetiger TastenDruecker;

    private int[,] TasPos ={{940, 380},     // --- Taste 0
                            {820, 380},     // --- Taste 1
                            {820, 600},     // --- Taste 2
                            {820, 780},     // --- Taste 3
                            {660, 380},     // --- Taste 4
                            {660, 600},     // --- Taste 5
                            {660, 780},     // --- Taste 6
                            {530, 380},     // --- Taste 7
                            {530, 600},     // --- Taste 8
                            {530, 780},     // --- Taste 9
                            {940, 1020},    // --- Plus
                            {940, 780},     // --- Gleich
                            {940, 200}      // --- Clear
                    };

    public frmMain() {
      InitializeComponent();
      RobData[] PosDaten = new RobData[] { 
                           new RobData(Mot.M1,950),new RobData(Mot.M2,1050)};
      Position = new RobMotors(fc, true, Ctr.Main, PosDaten);
      Position.ChangedPosition+=new RobMotors.PositionS(Position_ChangedPosition);
      TastenDruecker = new Betaetiger(fc, Ctr.Main, Mot.M3, Unv.I3, 555);
    }

    private delegate void LabelText();
    private void Position_ChangedPosition(RobMotors sender, int[] Pos) {
      lblPosition.Invoke(new LabelText(delegate() {
        lblPosition.Text = Pos[0].ToString() + " - " + Pos[1].ToString();
      }));
    }
    public class Betaetiger : Motor {
      private PushButton endTaster;
      private FishControl fc;
      private int upTime;
      public Betaetiger(FishControl fc, Ctr ctrId, Mot motName, Unv endTaster, int upTime)
        : base(fc, ctrId, motName) {
        this.fc = fc;
        this.upTime = upTime;
        this.endTaster = new PushButton(fc, ctrId, endTaster);
      }
      public void Druecke() {
        this.Down();
        endTaster.WaitForTrue();
        this.Up();
        fc.Pause(upTime);
        this.Off();
      }
    }

    private void cmdAction_Click(object sender, EventArgs e) {
      int OP1, OP2;
      int i, ptrTas;
      int[] Tasten = new int[24];

      try {
        if (cmdAction.Text == "START") {
          // --- Auf StartPosition fahren ---
          fc.Connect(txtCom.Text);
          fc.StartEvents();
          cmdEnde.Text = "HALT";
          
          lblStatus.Text = "Fährt auf Home-Position";
          TastenDruecker.Druecke();
          Position.StartHome();
          Position.WaitForDone();
          Position.MoveTo(TasPos[12, 0], TasPos[12, 1]);
          TastenDruecker.Druecke();

          lblStatus.Text = "Gestartet";
          cmdEnde.Text   = "ENDE";
          cmdAction.Text = "LÖSUNG";
          cmdEnde.Focus();
        }
        else if (cmdAction.Text == "LÖSUNG") {
          // --- Lösung suchen ----
          OP1 = Convert.ToInt32(txtOP1.Text);
          OP2 = Convert.ToInt32(txtOP2.Text);
          
          ptrTas = 0; Tasten[ptrTas++] = 12;  // --- Clear

          // --- Ziffernpositionen für ersten Operanden
          for (i = 0; i < txtOP1.Text.Length; i++)
            Tasten[ptrTas++] = Convert.ToInt32(txtOP1.Text.Substring(i, 1));
          
          Tasten[ptrTas++] = 10;              // --- Addition

          // --- Ziffernpositionen für zweiten Operanden
          for (i = 0; i < txtOP2.Text.Length; i++)
            Tasten[ptrTas++] = Convert.ToInt32(txtOP2.Text.Substring(i, 1));
          
          Tasten[ptrTas++] = 11;              // --- Taste =
          
          // --- Anfahren der erforderlichen Tasten ---
          cmdEnde.Text = "HALT"; lblStatus.Text = "Kontrolle mit dem Taschenrechner";
          for (i = 0; i < ptrTas; i++) {
            Position.MoveTo(TasPos[Tasten[i], 0], TasPos[Tasten[i], 1]);
            TastenDruecker.Druecke();
          }
          // --- Anzeige korrektes Ergebnis ---
          txtErgebnis.Text = Convert.ToString(OP1 + OP2);
          lblStatus.Text = "OK, das war's";
          cmdEnde.Text = "ENDE";
        }
      }
      catch (FishFaceException txe) {
        MessageBox.Show(txe.Message, this.Text,
          MessageBoxButtons.OK, MessageBoxIcon.Stop);
      }
      catch (Exception ee) {
        lblStatus.Text = ee.Message;
      }
    }

    private void cmdEnde_Click(object sender, EventArgs e) {
      if (cmdEnde.Text == "HALT") fc.NotHalt = true;
      else {
        fc.DisConnect();
        this.Close();
      }
    }

    private void formClosing(object sender, FormClosingEventArgs e) {
      if (cmdEnde.Text == "HALT") e.Cancel = true;
    }
  }
}